----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    16:21:48 02/06/2008 
-- Design Name: 
-- Module Name:    debouncer - Behavioral 
-- Project Name: 
-- Target Devices: Spartan3E
-- Tool versions: ISE 9.2.04i
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity DEBOUNCER is
	generic	(
					--Bitbreite des Sample- Zhlers
					SAMPLE_BIT_WIDTH	:	positive := 4
				);
	port		(
					--Systemtakt
					CLK					: in	std_logic;
					--Synchroner Reseteingang (active high)
					RESET					: in	bit;
					--Clock enable (hier kann der Systemtakt auf den Sample- Takt verringert werden)
					CE						: in	bit;
					--Eingang fr den prellenden Taster/Schalter
					BOUNCING_INPUT		: in	bit;
					--hoffentlich prellfreier Ausgang
					DEBOUNCED_OUTPUT	: out	bit
				);
end DEBOUNCER;

architecture Behavioral of DEBOUNCER is

--States der FSM
type FSM1_STATES is	(
								LOW_STATE,
								BOUNCING_UP_START,
								BOUNCING_UP,
								HIGH_STATE,
								BOUNCING_DOWN_START,
								BOUNCING_DOWN
							);

signal	FSM1_STATE, FSM1_NEXT_STATE	: FSM1_STATES;
--synchronisierter Eingang
signal	SAMPLED_KEY							: bit;
--Steuersignale fr den Sample- Zhler
signal	BOUNCE_COUNT_EN					: bit;
signal	BOUNCE_COUNT_CLEAR				: bit;
--interne Ausgnge der FSM (asynchron)
signal	BOUNCE_COUNT_EN_i					: bit;
signal	BOUNCE_COUNT_CLEAR_i				: bit;
signal	DEBOUNCED_OUTPUT_i				: bit;
--Zhlvariable fr die Samples
signal	COUNTVAR								: std_logic_vector(SAMPLE_BIT_WIDTH downto 0);

--Zhler mit Reset, Clock-Enable, Clear und Enable als Komponente
component COUNTER_RST_CE_CLR_EN is
	generic	(
					--Bitbreite des Zhlers
					BIT_WIDTH	:	positive
				);
	port		(
					--Systemtakt
					CLK		: in	std_logic;
					--Synchroner Reseteingang (active high)
					RESET		: in	bit;
					--Clock enable (um den Zhler unter dem Systemtakt zu betreiben)
					CE			: in	bit;
					--Systemtaktsynchroner clear- Eingang
					CLR		: in	bit;
					--Enable Eingang
					EN			: in	bit;
					--Ausgang
					OUTPUT	: out	std_logic_vector((BIT_WIDTH - 1) downto 0)
				);
end component;

for I_1: COUNTER_RST_CE_CLR_EN use entity WORK.COUNTER_RST_CE_CLR_EN(UP_COUNTER);

begin

	--Zhler wird benutzt um die Anzahl gleicher, hintereinanderfolgender Samples zu zhlen.
	I_1: COUNTER_RST_CE_CLR_EN
										generic map
														(
															BIT_WIDTH => (SAMPLE_BIT_WIDTH + 1)
														)
										port map		(
															CLK => CLK,
															RESET => RESET,
															CE => CE,
															CLR => BOUNCE_COUNT_CLEAR,
															EN => BOUNCE_COUNT_EN,
															OUTPUT => COUNTVAR
														);

	--Synchronisieren (samplen) des prellenden Eingangs
	KEY_SAMPLING: process(CLK)
	begin
		if rising_edge(CLK) then
			if RESET = '1' then
				SAMPLED_KEY <= '0';
			elsif CE = '1' then
				SAMPLED_KEY <= BOUNCING_INPUT;
			end if;
		end if;
	end process KEY_SAMPLING;

	--Synchronisieren der FSM- Signale
	FSM1_SYNC: process(CLK)
	begin
		if rising_edge(CLK) then
		
			if RESET = '1' then
				FSM1_STATE <= LOW_STATE;
				DEBOUNCED_OUTPUT		<= '0';
				BOUNCE_COUNT_EN		<= '0';
				BOUNCE_COUNT_CLEAR	<= '1';
			else
				FSM1_STATE <= FSM1_NEXT_STATE;
				DEBOUNCED_OUTPUT		<= DEBOUNCED_OUTPUT_i;
				BOUNCE_COUNT_EN		<= BOUNCE_COUNT_EN_i;
				BOUNCE_COUNT_CLEAR	<= BOUNCE_COUNT_CLEAR_i;
			end if;
		end if;
	end process FSM1_SYNC;
	
	--Die Ausgnge der FSM werden nach deren State gesetzt
	FSM1_OUTPUT_DECODE: process (FSM1_STATE)
   begin
		
		case FSM1_STATE is
		
			when BOUNCING_UP_START =>
				--hier wird der Ausgang schon auf high gesetzt (Taste wurde ja nunmal gedrckt)
				--jedoch der Samples Zhler wieder zurckgesetzt, da der Taster prellte
				DEBOUNCED_OUTPUT_i	<=	'1';
				BOUNCE_COUNT_EN_i		<= '0';
				BOUNCE_COUNT_CLEAR_i	<= '1';
				
			when BOUNCING_UP =>
				--hier ist der Ausgang auf high gesetzt und der Sample- Zhler luft
				--(bis jetzt waren also alle Samples vom gleichen Pegel)
				BOUNCE_COUNT_EN_i		<= '1';
				BOUNCE_COUNT_CLEAR_i	<= '0';
				DEBOUNCED_OUTPUT_i	<=	'1';
				
			when HIGH_STATE =>
				--hier ist der Ausgang natrlich auch high, der Zhler ist wieder gestoppt
				--und gelscht
				DEBOUNCED_OUTPUT_i	<=	'1';
				BOUNCE_COUNT_EN_i		<= '0';
				BOUNCE_COUNT_CLEAR_i	<= '1';
				
			when BOUNCING_DOWN =>
				--hier ist der Ausgang natrlich low (Taster wurde losgelassen), der Zhler luft
				BOUNCE_COUNT_EN_i		<= '1';
				BOUNCE_COUNT_CLEAR_i	<= '0';
				DEBOUNCED_OUTPUT_i	<=	'0';
				
			when others =>
				--bei jedem anderen Zustand wird der Ausgang auf low gesetzt und der Zhler
				--gestoppt und gelscht
				DEBOUNCED_OUTPUT_i	<=	'0';
				BOUNCE_COUNT_EN_i		<= '0';
				BOUNCE_COUNT_CLEAR_i	<= '1';
			end case;
		
   end process FSM1_OUTPUT_DECODE;
	
	--Die eigentliche FSM
	FSM1_NEXT_STATE_DECODE: process(FSM1_STATE, SAMPLED_KEY, COUNTVAR)
	begin

		case FSM1_STATE is
		
			when LOW_STATE =>
				--wird in diesem Zustand ein Highpegel am Taster abgetastet
				--(Taster gedrckt), so wird auf BOUNCING_UP_START gesetzt, 
				--ansonsten bleibt dieser Zustand aktuell
				if SAMPLED_KEY = '1' then
					FSM1_NEXT_STATE <= BOUNCING_UP_START;
				else
					FSM1_NEXT_STATE <= LOW_STATE;
				end if;
			
			when BOUNCING_UP_START =>
				--Dieser Zustand dient nur dazu im Ausgangsschaltnetz den Zhler
				--neu zu starten- daher wird immer direkt auf BOUNCING_UP weitergeschaltet
				FSM1_NEXT_STATE <= BOUNCING_UP;
			
			when BOUNCING_UP =>
				--wird in diesem Zustand ein high abgetastet, dann wird berprft,
				--ob das MSB des Samplezhlers eingeschaltet ist, wenn dem so ist wird in den
				--HIGH_STATE geschaltet, ansonsten bleibt der aktuelle Zustand.
				--Wird ein low abgetastet, so wird der Zustand auf BOUNCING_UP_START
				--zurck gesetzt (Taster prellt)
				if SAMPLED_KEY = '1' then
					if COUNTVAR(SAMPLE_BIT_WIDTH) = '1' then
						FSM1_NEXT_STATE <= HIGH_STATE;
					else
						FSM1_NEXT_STATE <= BOUNCING_UP;
					end if;
				else
					FSM1_NEXT_STATE <= BOUNCING_UP_START;
				end if;
				
			when HIGH_STATE =>
				--wird in diesem Zustand ein low abgetastet (Taster losgelassen), so wird auf 
				--BOUNCING_DOWN_START geschaltet. Ansonsten bleibt dieser Zustand aktuell.
				if SAMPLED_KEY = '0' then
					FSM1_NEXT_STATE <= BOUNCING_DOWN_START;
				else
					FSM1_NEXT_STATE <= HIGH_STATE;
				end if;
			
			when BOUNCING_DOWN_START =>
				--Dieser Zustand dient nur dazu im Ausgangsschaltnetz den Zhler
				--neu zu starten- daher wird immer direkt auf BOUNCING_DOWN weitergeschaltet
				FSM1_NEXT_STATE <= BOUNCING_DOWN;
			
			when BOUNCING_DOWN =>
				--wird in diesem Zustand ein high abgetastet, dann wird zu BOUNCING_DOWN_START
				--zurck geschaltet (Taster prellt). Ansonsten wird berprft,
				--ob das MSB des Samplezhlers eingeschaltet ist, wenn dem so ist wird in den
				--LOW_STATE geschaltet, ansonsten bleibt der aktuelle Zustand.
				if SAMPLED_KEY = '1' then
					FSM1_NEXT_STATE <= BOUNCING_DOWN_START;
				else
					if COUNTVAR(SAMPLE_BIT_WIDTH) = '1' then
						FSM1_NEXT_STATE <= LOW_STATE;
					else
						FSM1_NEXT_STATE <= BOUNCING_DOWN;
					end if;
				end if;
				
			when others =>
				--wird durch irgendwelche Einflsse die FSM gestrt, dann wird automatisch in
				--den LOW_STATE geschaltet
				FSM1_NEXT_STATE <= LOW_STATE;
				
		end case;
		
	end process FSM1_NEXT_STATE_DECODE;



end Behavioral;

